from ..power1.physics import PhysicSprite

class Player(PhysicSprite):
    def __init__(self, app, x, y, width=40, height=60, color="blue", speed=5,
                 left="left", right="right", jump="space"):
        super().__init__(app, x, y, width, height, color)
        if not isinstance(speed, (int, float)):
            raise TypeError(f"speed must be a number, got {type(speed).__name__}")
        if speed < 0:
            raise ValueError(f"speed must be positive, got {speed}")
        self.speed = speed
        self.key_left = left
        self.key_right = right
        self.key_jump = jump

    def handle_input(self):
        if self.pgs.key(self.key_left):
            self.vel_x = -self.speed
        elif self.pgs.key(self.key_right):
            self.vel_x = self.speed
        else:
            self.vel_x = 0
        if self.pgs.key(self.key_jump):
            self.jump()

    def tick(self):
        self.handle_input()


try:
 try:
    from ....pygame import*
 except ImportError:
    from pygame import*
except ImportError:
    raise ImportError("pygame or pygame-ce is required Install it with pip install pygame or pip install pygame-ce")    

class SSprites(sprite.Sprite):
    def __init__(self, pgs_instance, x, y, image_path=None, source=None):
        super().__init__()
        if not hasattr(pgs_instance, 'screen') or not hasattr(pgs_instance, 'objects'):
            raise TypeError(f"'{pgs_instance}' is not a valid pygames instance.")
        if not isinstance(x, (int, float)) or not isinstance(y, (int, float)):
            raise TypeError(f"x and y must be numbers, got x={type(x).__name__}, y={type(y).__name__}")
        self.pgs = pgs_instance
        if source:
            if isinstance(source, Surface):
                self.image = source
            else:
                try:
                    self.image = image.load(source).convert_alpha()
                except:
                    self.Error2(source)
        elif image_path:
            try:
                self.image = image.load(image_path).convert_alpha()
            except:
                self.Error2(image_path)
        else:
            self.image = Surface((50, 50))
            self.image.fill("white")

        self.rect = self.image.get_rect(topleft=(x, y))

    def Error2(self, name):
        raise NameError(f"The file '{name}' does not exist, check the file.")

    def draw(self):
        self.pgs.screen.blit(self.image, self.rect)

    def move(self, dx, dy):
        self.rect.x += dx
        self.rect.y += dy


from pygame import *
import sys

class pygames:
    def __init__(self, w=800, h=600, title="Pygame-S ", icon_path=None):
        init()
        font.init()
        self.screen_width = w
        self.screen_height = h
        self.screen = display.set_mode((w, h))
        self.set_title(title)
        if icon_path:
            self.set_icon(icon_path)
        self.clock = time.Clock()
        self.running = True
        self.font = font.SysFont("Arial", 30)
        self.sounds = {}
        self.images = {}
        mixer.init()
        self.objects = []
        self.solids = []
        self.score = 0
        self.score_active = False
        self.garbage_collection = False
        self.fps = 60

    def set_title(self, title):
        display.set_caption(str(title))

    def set_icon(self, path):
        icon = image.load(path)
        display.set_icon(icon)
           
    def Error1(self, name):
        raise NameError(f"The file {name} does not exist check the file.")
    
    def load_image(self, name, path):
        try:
            self.images[name] = image.load(path).convert_alpha()
        except:
            self.Error1(name)        

    def image(self, name, x, y, w=None, h=None):
        try:
           img = self.images.get(name)
           if img:              
               if w and h:
                  img = transform.scale(img, (w, h))
               self.screen.blit(img, (x, y))
        except:
            self.Error1(name)        

    def load_sound(self, name, path):
        try:
           self.sounds[name] = mixer.Sound(path)
        except:
              self.Error1(name)

    def play_sound(self, name):
        if name in self.sounds:
            self.sounds[name].play()
        else:
            self.Error1(name)   

    def start_score_counter(self):
        self.score_active = True

    def start_garbage_collecter(self):
        self.garbage_collection = True

    def clean_up(self):
        if self.garbage_collection:
            b = 150 
            self.objects = [
                o for o in self.objects 
                if o.rect.right > -b and o.rect.left < self.screen_width + b and
                   o.rect.bottom > -b and o.rect.top < self.screen_height + b
            ]

    def fill(self, color):
        self.screen.fill(color)

    def background(self, color):
        self.screen.fill(color)
    
    def start_loop(self):
        self.clean_up()
        for item in self.objects:
            if hasattr(item, 'apply_physics'):
                item.apply_physics(self.solids)
            item.draw()
        if self.score_active:
            text_surface = self.font.render(f"Score: {self.score}", True, "white")
            self.screen.blit(text_surface, (10, 10))
    
    def show_score(self, x=10, y=10, color="white"):
      score_surf = self.font.render(f"Score: {self.score}", True, color)
      self.screen.blit(score_surf, (x, y))        
        
    def start(self, *objects): 
        for item in objects:
            if item not in self.objects:
                self.objects.append(item)
    
    def make_solid(self, *objects):
        for item in objects:
            if item not in self.solids:
                self.solids.append(item)
                if item not in self.objects:
                    self.objects.append(item)
                         
    def key(self, name):
        keys = key.get_pressed()
        name = name.lower()
        special_keys = {
            "space": K_SPACE, "esc": K_ESCAPE, "up": K_UP,
            "down": K_DOWN, "left": K_LEFT, "right": K_RIGHT,
            "enter": K_RETURN, "shift": K_LSHIFT, "ctrl": K_LCTRL,
            "tab": K_TAB, "backspace": K_BACKSPACE
        }
        if name in special_keys:
            return keys[special_keys[name]]
        try:
            key_const = globals().get(f"K_{name.upper()}")
            return keys[key_const]
        except AttributeError:
            return False

    def zoom(self, factor):
        self.screen_width = int(self.screen_width * factor)
        self.screen_height = int(self.screen_height * factor)
        self.screen = display.set_mode((self.screen_width, self.screen_height))

    def speed(self, fps):
        self.fps= fps
   
    def update(self):
        display.flip()
        self.clock.tick(self.fps)
  
    def is_running(self):
        for e in event.get():
            if e.type == QUIT:
                self.running = False 
        return self.running         

    def mainloop(self, game_logic):
        while self.running: 
            self.is_running()
            game_logic()
            self.start_loop() 
            self.update()
        quit()
        sys.exit()


from setuptools import setup, find_packages 

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="pygames-simplified",
    version="1.8.1",
    description="PyGame Simplified — A beginner-friendly wrapper around Pygame",
    long_description=long_description,
    long_description_content_type="text/markdown",
    package_dir={'pygames': '.', 'pygames.pygames_engine': 'pygames_engine'},
    packages=['pygames', 'pygames.pygames_engine',
          'pygames.pygames_engine.engines',
          'pygames.pygames_engine.engines.power1',
          'pygames.pygames_engine.engines.power2',
          'pygames.pygames_engine.utils'],
    install_requires=[
        "pygame-ce>=2.0.0",
    ],
    python_requires=">=3.7",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Operating System :: OS Independent",
        "Topic :: Games/Entertainment",
        "Intended Audience :: Education",
        "Intended Audience :: Developers",
    ],
)
project_urls={
    "Source": "https://github.com/manupolice12-sketch/pygames",
    "Bug Tracker": "https://github.com/manupolice12-sketch/pygames/issues",
},


import time
import math
import pygame

class Animator:
    def __init__(self, target, animation_speed=0.1):
        self.target = target
        self.animation_speed = animation_speed
        self.current_frame = 0
        self.last_update_time = time.time()
        self.base_y = getattr(target, 'y', 0)

    def refresh(self):
        if not hasattr(self.target, 'animations'):
            return
            
        state = self.target.state
        if state not in self.target.animations:
            return

        current_time = time.time()
        
        if current_time - self.last_update_time > self.animation_speed:
            frames = self.target.animations[state]
            self.current_frame = (self.current_frame + 1) % len(frames)
            self.target.image = frames[self.current_frame]
            self.last_update_time = current_time

    def hover(self, amplitude=5, speed=5):
        self.target.y = self.base_y + math.sin(time.time() * speed) * amplitude

    def rotate_loop(self, speed=100):
        angle = (time.time() * speed) % 360
        self.target.image = pygame.transform.rotate(self.target.image, angle)

    def set_speed(self, speed):
        self.animation_speed = speed


try:
    from pygame import *
except ImportError:
    raise ImportError("pygame or pygame-ce is required. Install it with: pip install pygame-ce")
from ...utils.object_manager import SSprites

class PhysicSprite(SSprites):
    def __init__(self, app, x, y, width=50, height=50, color="red"):
        super().__init__(app, x, y)
        if not isinstance(width, (int, float)) or not isinstance(height, (int, float)):
            raise TypeError(f"width and height must be numbers, got width={type(width).__name__}, height={type(height).__name__}")
        self.image = Surface((width, height))
        try:
            self.image.fill(color)
        except:
              raise NameError(f"The color {color} does not exist please check the color you typed")    
        self.rect = self.image.get_rect(topleft=(x, y))
        self.vel_y = 0
        self.vel_x = 0
        self.gravity = 0.8
        self.max_fall_speed = 20
        self.on_ground = False

    def apply_physics(self, solids=None):
        if solids is None:
            solids = []
        self.vel_y += self.gravity
        if self.vel_y > self.max_fall_speed:
            self.vel_y = self.max_fall_speed
        self.rect.y += self.vel_y
        self.on_ground = False
        for solid in solids:
            if self.rect.colliderect(solid.rect):
                if self.vel_y > 0:
                    self.rect.bottom = solid.rect.top
                    self.vel_y = 0
                    self.on_ground = True
                elif self.vel_y < 0:
                    self.rect.top = solid.rect.bottom
                    self.vel_y = 0
        self.rect.x += self.vel_x
        for solid in solids:
            if self.rect.colliderect(solid.rect):
                if self.vel_x > 0:
                    self.rect.right = solid.rect.left
                elif self.vel_x < 0:
                    self.rect.left = solid.rect.right
                self.vel_x = 0
        if self.rect.bottom > self.pgs.screen_height:
            self.rect.bottom = self.pgs.screen_height
            self.vel_y = 0
            self.on_ground = True

    def jump(self, force=15):
        if not isinstance(force, (int, float)):
            raise TypeError(f"jump force must be a number, got {type(force).__name__}")
        if self.on_ground:
            self.vel_y = -force
            self.on_ground = False